@@ -72,6 +72,8 @@
 #' Currently undocumented
 #' @param normalize
 #' String: Name of normalization for all non-indicator covariates (possible values: stdev, max, median)
+#' @param floatingPoint
+#' Integer: Floating-point representation size (32 or 64)
 #'
 #' @return
 #' A list that contains a Cyclops model data object pointer and an operation duration
@@ -96,12 +98,16 @@
 #' @export
 createCyclopsData <- function(formula, sparseFormula, indicatorFormula, modelType,
                               data, subset = NULL, weights = NULL, offset = NULL, time = NULL, pid = NULL, y = NULL, type = NULL, dx = NULL,
-                              sx = NULL, ix = NULL, model = FALSE, normalize = NULL, method = "cyclops.fit") {
+                              sx = NULL, ix = NULL, model = FALSE, normalize = NULL,
+                              floatingPoint = 64,
+                              method = "cyclops.fit") {
     cl <- match.call() # save to return
     mf.all <- match.call(expand.dots = FALSE)
 
     if (!.isValidModelType(modelType)) stop("Invalid model type.")
 
+    .checkFloatingPoint(floatingPoint)
+
     hasIntercept <- FALSE
     colnames <- NULL
 
@@ -264,6 +270,9 @@ createCyclopsData <- function(formula, sparseFormula, indicatorFormula, modelTyp
             if (!missing(type)) {
                 type <- type[sortOrder]
             }
+            if (!missing(weights)) {
+                weights <- weights[sortOrder]
+            }
             time <- time[sortOrder]
             dx <- dx[sortOrder, ]
             if (class(dx) == "numeric") {
@@ -328,7 +337,8 @@ createCyclopsData <- function(formula, sparseFormula, indicatorFormula, modelTyp
         pid <- c(1:length(y)) # TODO Should not be necessary
     }
 
-    md <- .cyclopsModelData(pid, y, type, time, dx, sx, ix, modelType, useTimeAsOffset, numTypes)
+    md <- .cyclopsModelData(pid, y, type, time, dx, sx, ix, modelType, useTimeAsOffset, numTypes,
+                            floatingPoint)
     result <- new.env(parent = emptyenv())
     result$cyclopsDataPtr <- md$data
     result$modelType <- modelType
@@ -350,6 +360,7 @@ createCyclopsData <- function(formula, sparseFormula, indicatorFormula, modelTyp
     }
 
     result$sortOrder <- sortOrder
+    result$weights <- weights
 
     if (identical(method, "debug")) {
         result$debug <- list()
@@ -373,6 +384,12 @@ createCyclopsData <- function(formula, sparseFormula, indicatorFormula, modelTyp
     result
 }
 
+.checkFloatingPoint <- function(floatingPoint) {
+      if (floatingPoint != 64 && floatingPoint != 32) {
+          stop("Invalid floating point precision")
+      }
+}
+
 .normalizeCovariates <- function(cyclopsData, type) {
     scale <- .cyclopsNormalizeCovariates(cyclopsData, type)
 
@@ -822,7 +839,8 @@ finalizeSqlCyclopsData <- function(object,
 }
 
 #' @keywords internal
-createSqlCyclopsData <- function(modelType, control) {
+createSqlCyclopsData <- function(modelType, control,
+                                 floatingPoint = 64) {
     cl <- match.call() # save to return
 
     if (!.isValidModelType(modelType)) stop("Invalid model type.")
@@ -833,7 +851,7 @@ createSqlCyclopsData <- function(modelType, control) {
         noiseLevel <- control$noiseLevel
     }
 
-    sql <- .cyclopsNewSqlData(modelType, noiseLevel)
+    sql <- .cyclopsNewSqlData(modelType, noiseLevel, floatingPoint)
     result <- new.env(parent = emptyenv()) # TODO Remove code duplication with two functions above
     result$cyclopsDataPtr <- sql$cyclopsDataPtr
     result$modelType <- modelType
@@ -942,6 +960,7 @@ print.cyclopsData <- function(x, show.call=TRUE ,...) {
         if (.cyclopsGetHasOffset(x)) {
             cat("    Offset: ", .cyclopsGetMeanOffset(x), " (mean)\n", sep = "")
         }
+        cat("   FP size:", getFloatingPointSize(x))
     } else {
         cat("\nObject is no longer or improperly initialized.\n")
     }
