@@ -28,7 +28,7 @@ public:
 namespace OutputHelper {
 struct NoMissingPolicy {
 	// No API at moment
-	bool isMissing(real value) {
+	bool isMissing(double value) {
 		return false;
 	}
 
@@ -144,7 +144,7 @@ private:
 template <typename DerivedFormat, typename Missing = OutputHelper::NoMissingPolicy>
 class BaseOutputWriter : public OutputWriter, Missing {
 public:
-	BaseOutputWriter(CyclicCoordinateDescent& _ccd, const ModelData& _data) :
+	BaseOutputWriter(CyclicCoordinateDescent& _ccd, const AbstractModelData& _data) :
 		OutputWriter(), ccd(_ccd), data(_data), delimitor(","), endl("\n") {
 		// Do nothing
 	}
@@ -210,7 +210,7 @@ protected:
 	}
 
 	CyclicCoordinateDescent& ccd;
-	const ModelData& data;
+	const AbstractModelData& data;
 	string delimitor;
 	string endl;
 };
@@ -220,7 +220,7 @@ protected:
 
 class DiagnosticsOutputWriter : public BaseOutputWriter<DiagnosticsOutputWriter> {
 public:
-	DiagnosticsOutputWriter(CyclicCoordinateDescent& ccd, const ModelData& data) :
+	DiagnosticsOutputWriter(CyclicCoordinateDescent& ccd, const AbstractModelData& data) :
 		BaseOutputWriter<DiagnosticsOutputWriter>(ccd, data) {
 		// Do nothing
 	}
@@ -298,7 +298,7 @@ private:
 
 class PredictionOutputWriter : public BaseOutputWriter<PredictionOutputWriter> {
 public:
-	PredictionOutputWriter(CyclicCoordinateDescent& ccd, const ModelData& data) :
+	PredictionOutputWriter(CyclicCoordinateDescent& ccd, const AbstractModelData& data) :
 		BaseOutputWriter<PredictionOutputWriter>(ccd, data) {
 		// Do nothing
 	}
@@ -335,7 +335,7 @@ public:
 	}
 
 private:
-	std::vector<real> predictions; // TODO Should be double
+	std::vector<double> predictions;
 };
 
 // struct ProfileInformation {
@@ -353,7 +353,7 @@ private:
 
 class EstimationOutputWriter : public BaseOutputWriter<EstimationOutputWriter> {
 public:
-	EstimationOutputWriter(CyclicCoordinateDescent& ccd, const ModelData& data) :
+	EstimationOutputWriter(CyclicCoordinateDescent& ccd, const AbstractModelData& data) :
 		BaseOutputWriter<EstimationOutputWriter>(ccd, data), withProfileBounds(false) {
 		// Do nothing
 	}
@@ -399,7 +399,7 @@ public:
 	template <typename Stream>
 	void writeRow(Stream& out, OutputHelper::RowInformation& rowInfo) {
 	    if (rowInfo.currentRow > 0 || !data.getHasOffsetCovariate()) {
-            out.addValue(data.getColumn(rowInfo.currentRow).getNumericalLabel()).addDelimitor();
+            out.addValue(data.getColumnNumericalLabel(rowInfo.currentRow)).addDelimitor();
             out.addValue(ccd.getBeta(rowInfo.currentRow));
             if (withProfileBounds && informationList[rowInfo.currentRow].defined) {
                 // TODO Delegate to friend of ProfileInformation
